@@ -63,6 +63,13 @@ conversion_imageinfo_changed( Conversion *conv )
 	return( NULL );
 }
 
+static void
+conversion_area_changed( Conversion *conv, Rect *dirty )
+{
+	g_signal_emit( G_OBJECT( conv ), 
+		conversion_signals[SIG_AREA_CHANGED], 0, dirty );
+}
+
 static void
 conversion_dispose( GObject *gobject )
 {
@@ -216,7 +223,6 @@ conversion_render_idle_cb( gpointer data )
 {
 	ConversionUpdate *update = (ConversionUpdate *) data;
 	Conversion *conv = update->conv;
-	Rect image;
 
 	/* Must be a valid conversion, must be for the image that that 
 	 * conversion is still using for display.
@@ -238,22 +244,16 @@ conversion_render_idle_cb( gpointer data )
 		update->area.width, update->area.height );
 #endif /*DEBUG*/
 
-	conversion_disp_to_im_rect( conv, &update->area, &image );
-
-	/* If we're zoomed out, expand the margins to allow for rounding.
-	 */
-	if( update->conv->mag < 0 ) {
-		image.width += 1;
-		image.height += 1;
-	}
-
 	/* We need to invalid the main image too, since those regions will
 	 * have black in from the failed first calc.
+	 *
+	 * im_render() can't do this invalidate for us, it needs to be done
+	 * from the main loop.
 	 */
 	im_invalidate( conv->mask );
 	im_invalidate( imageinfo_get( FALSE, conv->display_ii ) );
 
-	imageinfo_area_changed( conv->ii, &image );
+	conversion_area_changed( conv, &update->area );
 
 	g_free( update );
 
@@ -306,7 +306,7 @@ conversion_make_display( Conversion *conv, IMAGE *in, IMAGE **mask_out )
 			FIXME ... look for pyramid TIFFs here
 
 		 */
-		IMAGE *t = im_open_local( out, "conv:1", "p" );
+		IMAGE *t = im_open_local( out, "conv:s", "p" );
 
 		/* Don't shrink by more than the image size (ie. to less than
 		 * 1 pixel).
@@ -314,9 +314,17 @@ conversion_make_display( Conversion *conv, IMAGE *in, IMAGE **mask_out )
 		int xshrink = IM_MIN( -conv->mag, in->Xsize );
 		int yshrink = IM_MIN( -conv->mag, in->Ysize );
 
-		if( !t || im_subsample( in, t, xshrink, yshrink ) ) {
-			im_close( out );
-			return( NULL );
+		if( DISPLAY_THUMBNAIL_HQ ) {
+			if( !t || im_shrink( in, t, xshrink, yshrink ) ) {
+				im_close( out );
+				return( NULL );
+			}
+		}
+		else {
+			if( !t || im_subsample( in, t, xshrink, yshrink ) ) {
+				im_close( out );
+				return( NULL );
+			}
 		}
 
 		in = t;
@@ -325,7 +333,7 @@ conversion_make_display( Conversion *conv, IMAGE *in, IMAGE **mask_out )
 	/* Zoom, if necessary. 
 	 */
 	if( conv->mag > 1 ) {
-		IMAGE *t = im_open_local( out, "conv:1", "p" );
+		IMAGE *t = im_open_local( out, "conv:z", "p" );
 
 		if( !t || im_zoom( in, t, conv->mag, conv->mag ) ) {
 			im_close( out );
@@ -344,7 +352,7 @@ conversion_make_display( Conversion *conv, IMAGE *in, IMAGE **mask_out )
 		}
 	}
 	else {
-		IMAGE *mask = im_open_local( out, "conv:1a", "p" );
+		IMAGE *mask = im_open_local( out, "conv:r", "p" );
 
 		if( im_render_priority( in, out, mask, 
 			conv->tile_size, conv->tile_size, 
@@ -713,7 +721,7 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 
 		/* im_msb() only works for the int formats.
 		 */
-		if( im_isint( in ) ) {
+		if( vips_bandfmt_isint( in->BandFmt ) ) {
 			if( im_msb( in, t[0] ) ) {
 				im_close( out );
 				return( NULL );
@@ -734,7 +742,7 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 	if( in->BandFmt != IM_BANDFMT_UCHAR ) {
 		IMAGE *t = im_open_local( out, "conv:1", "p" );
 
-		if( !t || im_clip( in, t ) ) {
+		if( !t || im_clip2fmt( in, t, IM_BANDFMT_UCHAR ) ) {
 			im_close( out );
 			return( NULL );
 		}
@@ -1048,7 +1056,7 @@ conversion_init( Conversion *conv )
 	/* Default tile size ... OK for image display, too big for
 	 * thumbnails.
 	 */
-	conv->tile_size = 128;
+	conv->tile_size = 64;
 
 	conv->underlay = emptyrect;
 	conv->image = emptyrect;
@@ -1120,6 +1128,8 @@ conversion_ii_changed_cb( Imageinfo *ii, Conversion *conv )
 	iobject_changed( IOBJECT( conv ) );
 }
 
+/* Something like a paint action on the ii.
+ */
 static void
 conversion_ii_area_changed_cb( Imageinfo *imageinfo, 
 	Rect *dirty, Conversion *conv )
@@ -1127,8 +1137,7 @@ conversion_ii_area_changed_cb( Imageinfo *imageinfo,
 	Rect repaint;
 
 	conversion_im_to_disp_rect( conv, dirty, &repaint );
-	g_signal_emit( G_OBJECT( conv ), 
-		conversion_signals[SIG_AREA_CHANGED], 0, &repaint );
+	conversion_area_changed( conv, &repaint );
 }
 
 /* Install a new image. 
@@ -1357,6 +1366,11 @@ void
 conversion_set_synchronous( Conversion *conv, gboolean synchronous )
 {
 	if( conv->synchronous != synchronous ) {
+#ifdef DEBUG
+		printf( "conversion_set_synchronous: %d", synchronous );
+		iobject_print( IOBJECT( conv ) );
+#endif /*DEBUG*/
+
 		conv->synchronous = synchronous;
 		if( conv->ii )
 			iobject_changed( IOBJECT( conv->ii ) );
