@@ -494,6 +494,15 @@ set_tooltip_generate( GtkWidget *widget,
 		tooltip_generate_attach( widget, gen );
 }
 
+/* Junk all tooltips, helps trim valgrind noise.
+ */
+void
+junk_tooltips( void )
+{
+	if( our_tooltips )
+		g_object_ref_sink( GTK_OBJECT( our_tooltips ) );
+}
+
 /* Set a GtkEditable.
  */
 void
@@ -890,7 +899,8 @@ filedrop_drag_data_received( GtkWidget *widget,
 #if !GLIB_CHECK_VERSION (2,0,0)
 		result = fdi->fn( fdi->client, sPath );
 #else
-		/* format changed with Gtk+1.3, use conversion */
+		/* format changed with Gtk+1.3, use conversion 
+		 */
 		pFrom = g_filename_from_uri( sPath, NULL, &error );
 		result = fdi->fn( fdi->client, pFrom );
 		g_free( pFrom );
