@@ -1,5 +1,4 @@
-/* Imagedisplay widget code ... display entire image, place this widget in a
- * scrolledwindow to get clipping/scrolling behaviour.
+/* Imagedisplay widget code ... display entire image, place this widget in a * scrolledwindow to get clipping/scrolling behaviour.
  */
 
 /*
@@ -56,114 +55,106 @@ static guint imagedisplay_signals[SIG_LAST] = { 0 };
 void
 imagedisplay_queue_draw_area( Imagedisplay *id, Rect *area )
 {
+#ifdef DEBUG_PAINT
+	printf( "imagedisplay_queue_draw_area: "
+		"left = %d, top = %d, width = %d, height = %d\n",
+		area->left, area->top, area->width, area->height );
+#endif /*DEBUG_PAINT*/
+
 	gtk_widget_queue_draw_area( GTK_WIDGET( id ),
 		area->left, area->top, area->width, area->height ); 
 }
 
-/* Does an area need painting? Test the mask for non-zero pixels.
- */
-static gboolean
-imagedisplay_needs_painting( Imagedisplay *id, Rect *area )
-{
-	Conversion *conv = id->conv;
-	guchar *buf;
-	int lsk;
-	int x, y;
-
-	/* No mask? Always paint.
-	 */
-	if( !conv->mreg )
-		return( TRUE );
-
-	/* If the mask is all zero, don't paint. This can leave old pixels on
-	 * the screen, but it does stop a lot of flicker and makes updates
-	 * look cleaner.
-	 */
-	if( im_prepare( conv->mreg, area ) )
-		return( TRUE );
-        buf = (guchar *) IM_REGION_ADDR( conv->mreg, area->left, area->top );
-        lsk = IM_REGION_LSKIP( conv->mreg );
-	for( y = 0; y < area->height; y++ ) {
-		for( x = 0; x < area->width; x++ )
-			if( buf[x] )
-				return( TRUE );
-		buf += lsk;
-	}
-
-	return( FALSE );
-}
-
 /* Repaint an area of the image.
  */
 static void
-imagedisplay_paint_image( Imagedisplay *id, GdkRectangle *expose )
+imagedisplay_paint_image( Imagedisplay *id, Rect *area )
 {
 	Conversion *conv = id->conv;
 
-	GdkRectangle clip;
-	GdkRectangle canvas;
-	Rect vclip;
 	guchar *buf;
 	int lsk;
 
-	if( !GTK_WIDGET( id )->window )
-		return;
-	if( !GTK_WIDGET_VISIBLE( id ) )
-		return;
-	if( !conv->ireg )
-		return;
-
-	/* Clip as much as we can.
-	 */
-	canvas.x = 0;
-	canvas.y = 0;
-	canvas.width = conv->canvas.width;
-	canvas.height = conv->canvas.height;
-	if( !gdk_rectangle_intersect( expose, &canvas, &clip ) )
-		return;
-
-	/* Pixels we need.
-	 */
-	vclip.left = clip.x;
-	vclip.top = clip.y;
-	vclip.width = clip.width;
-	vclip.height = clip.height;
-
-	/* Test for paintability.
-	 */
-	if( !imagedisplay_needs_painting( id, &vclip ) )
-		return;
-
 #ifdef DEBUG_PAINT
 	g_print( "imagedisplay_paint_image: at %d x %d, size %d x %d ",
-		clip.x, clip.y, clip.width, clip.height );
+		area->left, area->top, area->width, area->height );
 	gobject_print( G_OBJECT( id ) );
 #endif /*DEBUG_PAINT*/
 
-	if( im_prepare( conv->ireg, &vclip ) ) {
+	/* Request pixels. We ask the mask first, to get an idea of what's
+	 * currently in cache, then request tiles of pixels. We must always
+	 * request pixels, even if the mask is blank, because the request
+	 * will trigger a notify later which will reinvoke us.
+	 */
+	if( conv->mreg &&
+		im_prepare( conv->mreg, area ) ) {
 #ifdef DEBUG_PAINT
+		printf( "imagedisplay_paint_image: mask paint error\n" );
+		printf( "\t%s\n", im_error_buffer() );
 #endif /*DEBUG_PAINT*/
+
+		return;
+	}
+	if( im_prepare( conv->ireg, area ) ) {
+#ifdef DEBUG_PAINT
 		printf( "imagedisplay_paint_image: paint error\n" );
-		im_error_buffer();
+		printf( "\t%s\n", im_error_buffer() );
+#endif /*DEBUG_PAINT*/
+
 		im_error_clear();
 
 		return;
 	}
-	buf = (guchar *) IM_REGION_ADDR( conv->ireg, vclip.left, vclip.top );
-	lsk = IM_REGION_LSKIP( conv->ireg );
+
+	/* Is the mask all zero? Skip the paint.
+	 */
+	if( conv->mreg ) {
+		gboolean found;
+		int x, y;
+
+		buf = (guchar *) 
+			IM_REGION_ADDR( conv->mreg, area->left, area->top );
+		lsk = IM_REGION_LSKIP( conv->mreg );
+		found = FALSE;
+
+		for( y = 0; y < area->height; y++ ) {
+			for( x = 0; x < area->width; x++ )
+				if( buf[x] ) {
+					found = TRUE;
+					break;
+				}
+
+			if( found )
+				break;
+
+			buf += lsk;
+		}
+
+		if( !found ) {
+#ifdef DEBUG_PAINT
+			printf( "imagedisplay_paint_image: zero mask\n" );
+#endif /*DEBUG_PAINT*/
+
+			return;
+		}
+	}
 
 	/* Paint into window.
 	 */
+
+	buf = (guchar *) IM_REGION_ADDR( conv->ireg, area->left, area->top );
+	lsk = IM_REGION_LSKIP( conv->ireg );
+
 	if( conv->ireg->im->Bands == 3 )
 		gdk_draw_rgb_image( GTK_WIDGET( id )->window,
 			GTK_WIDGET( id )->style->white_gc,
-			clip.x, clip.y, clip.width, clip.height,
+			area->left, area->top, area->width, area->height,
 			GDK_RGB_DITHER_MAX,
 			buf, lsk );
 	else if( conv->ireg->im->Bands == 1 )
 		gdk_draw_gray_image( GTK_WIDGET( id )->window,
 			GTK_WIDGET( id )->style->white_gc,
-			clip.x, clip.y, clip.width, clip.height,
+			area->left, area->top, area->width, area->height,
 			GDK_RGB_DITHER_MAX,
 			buf, lsk );
 }
@@ -171,25 +162,25 @@ imagedisplay_paint_image( Imagedisplay *id, GdkRectangle *expose )
 /* Paint an area with the background pattern.
  */
 static void
-imagedisplay_paint_background( Imagedisplay *id, GdkRectangle *expose )
+imagedisplay_paint_background( Imagedisplay *id, Rect *expose )
 {
 #ifdef DEBUG_PAINT
 	g_print( "imagedisplay_paint_background: at %d x %d, size %d x %d\n",
-		expose->x, expose->y, expose->width, expose->height );
+		expose->left, expose->top, expose->width, expose->height );
 #endif /*DEBUG_PAINT*/
 
 	gdk_draw_rectangle( GTK_WIDGET( id )->window, 
 		id->back_gc, TRUE,
-		expose->x, expose->y, expose->width, expose->height );
+		expose->left, expose->top, expose->width, expose->height );
 }
 
 /* Paint areas outside the image.
  */
 static void
-imagedisplay_paint_background_clipped( Imagedisplay *id, GdkRectangle *expose )
+imagedisplay_paint_background_clipped( Imagedisplay *id, Rect *expose )
 {
 	Conversion *conv = id->conv;
-	GdkRectangle image, clip, area;
+	Rect clip;
 
 #ifdef DEBUG_PAINT
 	g_print( "imagedisplay_paint_background_clipped: canvas %d x %d\n",
@@ -200,19 +191,18 @@ imagedisplay_paint_background_clipped( Imagedisplay *id, GdkRectangle *expose )
 	 * everything to the right of the image, and everything strictly
 	 * below.
 	 */
-	image.x = 0;
-	image.y = 0;
-	image.width = conv->canvas.width;
-	image.height = conv->canvas.height;
-	if( gdk_rectangle_intersect( expose, &image, &clip ) ) {
+	im_rect_intersectrect( expose, &conv->canvas, &clip );
+	if( !im_rect_isempty( &clip ) ) {
+		Rect area;
+
 		area = *expose;
-		area.x = conv->canvas.width;
+		area.left = conv->canvas.width;
 		area.width -= clip.width;
 		if( area.width > 0 )
 			imagedisplay_paint_background( id, &area );
 
 		area = *expose;
-		area.y = conv->canvas.height;
+		area.top = conv->canvas.height;
 		area.width = clip.width;
 		area.height -= clip.height;
 		if( area.height > 0 )
@@ -223,21 +213,46 @@ imagedisplay_paint_background_clipped( Imagedisplay *id, GdkRectangle *expose )
 }
 
 static void
-imagedisplay_paint( Imagedisplay *id, GdkRectangle *area )
+imagedisplay_paint( Imagedisplay *id, Rect *area )
 {
+	Conversion *conv = id->conv;
+	const int tsize = conv->tile_size;
+
+	Rect clip;
+	int xs, ys;
+	int x, y;
+
+	/* Clip non-image parts of the expose.
+	 */
+	im_rect_intersectrect( area, &conv->canvas, &clip );
+	if( im_rect_isempty( &clip ) )
+		return;
+
 #ifdef DEBUG_PAINT
 	g_print( "imagedisplay_paint: at %d x %d, size %d x %d\n",
-		area->x, area->y, area->width, area->height );
+		clip.left, clip.top, clip.width, clip.height );
 #endif /*DEBUG_PAINT*/
 
-	/* Clear to background. Always do this, to make sure we paint 
-	 * outside the image area.
+	/* Round left/top down to the start tile.
 	 */
-	imagedisplay_paint_background_clipped( id, area );
+	xs = (clip.left / tsize) * tsize;
+	ys = (clip.top / tsize) * tsize;
 
-	/* Paint image.
+	/* Now loop painting image tiles.
 	 */
-	imagedisplay_paint_image( id, area );
+	for( y = ys; y < IM_RECT_BOTTOM( &clip ); y += tsize )
+		for( x = xs; x < IM_RECT_RIGHT( &clip ); x += tsize ) {
+			Rect tile;
+			Rect tile2;
+
+			tile.left = x;
+			tile.top = y;
+			tile.width = conv->tile_size;
+			tile.height = conv->tile_size;
+			im_rect_intersectrect( &tile, &clip, &tile2 );
+
+			imagedisplay_paint_image( id, &tile2 );
+		}
 }
 
 /* Expose signal handler.
@@ -246,19 +261,40 @@ static gint
 imagedisplay_expose( GtkWidget *widget, GdkEventExpose *event )
 {
 	Imagedisplay *id = IMAGEDISPLAY( widget );
+	Conversion *conv = id->conv;
+
 	GdkRectangle *rect;
 	int i, n;
 
 	if( !GTK_WIDGET_DRAWABLE( id ) ||
-		event->area.width == 0 || event->area.height == 0 )
+		event->area.width == 0 || 
+		event->area.height == 0 ||
+		!GTK_WIDGET( id )->window ||
+		!GTK_WIDGET_VISIBLE( id ) ||
+		!conv->ireg )
 		return( FALSE );
 
 	gdk_region_get_rectangles( event->region, &rect, &n );
 #ifdef DEBUG_PAINT
 	g_print( "imagedisplay_expose: %d rectangles\n", n ); 
 #endif /*DEBUG_PAINT*/
-	for( i = 0; i < n; i++ ) 
-		imagedisplay_paint( id, &rect[i] );
+	for( i = 0; i < n; i++ ) {
+		Rect area;
+
+		area.left = rect[i].x;
+		area.top = rect[i].y;
+		area.width = rect[i].width;
+		area.height = rect[i].height;
+
+		/* Clear to background. Always do this, to make sure we paint 
+		 * outside the image area.
+		 */
+		imagedisplay_paint_background_clipped( id, &area );
+
+		/* And paint pixels.
+		 */
+		imagedisplay_paint( id, &area );
+	}
 	g_free( rect );
 
         return( FALSE );
@@ -358,12 +394,6 @@ imagedisplay_real_conversion_changed( Imagedisplay *id )
 static void
 imagedisplay_real_area_changed( Imagedisplay *id, Rect *dirty )
 {
-#ifdef DEBUG_PAINT
-	g_print( "imagedisplay_real_area_changed: left = %d, top = %d, "
-		"width = %d, height = %d\n",
-		dirty->left, dirty->top, dirty->width, dirty->height );
-#endif /*DEBUG_PAINT*/
-
 	imagedisplay_queue_draw_area( id, dirty );
 }
 
@@ -481,12 +511,18 @@ imagedisplay_get_type( void )
 	return( type );
 }
 
-/* Conversion has changed ... resize and repaint.
+/* Conversion has changed ... repaint everything.
  */
 static void
 imagedisplay_conversion_changed_cb( Conversion *conv, Imagedisplay *id )
 {
+#ifdef DEBUG
+	printf( "imagedisplay_conversion_changed_cb: " );
+	gobject_print( G_OBJECT( id ) );
+#endif /*DEBUG*/
+
 	IMAGEDISPLAY_GET_CLASS( id )->conversion_changed( id );
+
 	g_signal_emit( G_OBJECT( id ), 
 		imagedisplay_signals[SIG_AREA_CHANGED], 0, &conv->canvas );
 }
@@ -497,6 +533,13 @@ static void
 imagedisplay_conversion_area_changed_cb( Conversion *conv, 
 	Rect *dirty, Imagedisplay *id )
 {
+#ifdef DEBUG
+	printf( "imagedisplay_conversion_area_changed_cb: " 
+		"left = %d, top = %d, width = %d, height = %d, ",
+		dirty->left, dirty->top, dirty->width, dirty->height );
+	gobject_print( G_OBJECT( id ) );
+#endif /*DEBUG*/
+
 	g_signal_emit( G_OBJECT( id ), 
 		imagedisplay_signals[SIG_AREA_CHANGED], 0, dirty );
 }
