@@ -83,10 +83,10 @@ most of the jobs above are pushed down into vips8 now ... except for
 
 /*
 #define DEBUG
-#define DEBUG_CHECK
 #define DEBUG_MAKE
-#define DEBUG_OPEN
 #define DEBUG_RGB
+#define DEBUG_CHECK
+#define DEBUG_OPEN
  */
 
 static iContainerClass *imageinfogroup_parent_class = NULL;
@@ -272,8 +272,8 @@ imageinfo_area_changed( Imageinfo *imageinfo, Rect *dirty )
 	g_assert( IS_IMAGEINFO( imageinfo ) );
 
 #ifdef DEBUG
-	printf( "imageinfo_area_changed: left = %d, top = %d, "
-		"width = %d, height = %d\n",
+	printf( "imageinfo_area_changed: "
+		"left = %d, top = %d, width = %d, height = %d\n",
 		dirty->left, dirty->top, dirty->width, dirty->height );
 #endif /*DEBUG*/
 
@@ -393,7 +393,7 @@ imageinfo_get_underlying( Imageinfo *imageinfo )
 	if( imageinfo->underlying )
 		return( imageinfo_get_underlying( imageinfo->underlying ) );
 	else
-		return(  imageinfo->im );
+		return( imageinfo->im );
 }
 
 /* Free up an undo fragment. 
@@ -470,8 +470,7 @@ static void
 imageinfo_finalize( GObject *gobject )
 {
 	Imageinfo *imageinfo = IMAGEINFO( gobject );
-	IMAGE *im = imageinfo_get_underlying( imageinfo );
-	gboolean isfile = im ? im_isfile( im ) : FALSE;
+	gboolean isfile = imageinfo->im ? im_isfile( imageinfo->im ) : FALSE;
 	char name[FILENAME_MAX];
 
 #ifdef DEBUG_MAKE
@@ -479,23 +478,21 @@ imageinfo_finalize( GObject *gobject )
 	imageinfo_print( imageinfo );
 #endif /*DEBUG_MAKE*/
 
-	if( imageinfo->dfile && isfile ) 
-		/* We must close before we delete to make sure we
-		 * get the desc file too ... save the filename.
+	if( imageinfo->dfile && isfile ) {
+		/* We must close before we delete ... save the filename.
 		 */
-		im_strncpy( name, im->filename, FILENAME_MAX - 5 );
+		im_strncpy( name, imageinfo->im->filename, FILENAME_MAX - 5 );
+
+#ifdef DEBUG_OPEN
+		printf( "imageinfo_destroy: unlinking \"%s\"\n", name );
+#endif /*DEBUG_OPEN*/
+	}
 
 	IM_FREEF( im_close, imageinfo->im );
 	IM_FREEF( im_close, imageinfo->mapped_im );
 	IM_FREEF( im_close, imageinfo->identity_lut );
 
 	if( imageinfo->dfile && isfile ) {
-#ifdef DEBUG_OPEN
-		printf( "imageinfo_destroy: unlinking \"%s\"\n", name );
-#endif /*DEBUG_OPEN*/
-
-		unlinkf( "%s", name );
-		strcpy( name + strlen( name ) - 1, "desc" );
 		unlinkf( "%s", name );
 		iobject_changed( IOBJECT( main_imageinfogroup ) );
 	}
@@ -673,7 +670,7 @@ imageinfo_proxy_eval( Imageinfoproxy *proxy )
 {
 	Imageinfo *imageinfo = proxy->imageinfo;
 
-	if( imageinfo ) 
+	if( imageinfo && imageinfo->im->time ) 
 		if( progress_update_percent( imageinfo->im->time->percent,
 			imageinfo->im->time->eta ) ) {
 			im_error( "nip2", _( "User cancelled operation" ) );
@@ -876,8 +873,8 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 		}
 
 #ifdef DEBUG_OPEN
-		printf( "imageinfo_open_image_input: "
-			"opened %s \"%s\"\n", format->name, filename );
+		printf( "imageinfo_open_image_input: opened %s \"%s\"\n", 
+			VIPS_OBJECT_CLASS( format )->nickname, filename );
 #endif /*DEBUG_OPEN*/
 	}
 
@@ -1126,53 +1123,12 @@ imageinfo_write( Imageinfo *imageinfo, const char *name )
 	return( TRUE );
 }
 
-/* Change an imageinfo to be a file, rather than a memory object. 
- */
-static gboolean
-imageinfo_file( Imageinfo *imageinfo )
-{
-	IMAGE *im;
-	char filename[FILENAME_MAX];
-
-	/* Check image type.
-	 */
-	if( !imageinfo_get( FALSE, imageinfo ) )
-		return( FALSE );
-	if( im_isfile( imageinfo_get( FALSE, imageinfo ) ) )
-		return( TRUE );
-
-	/* Save it.
-	 */
-	if( !temp_name( filename, "v" ) )
-		return( FALSE );
-	if( !imageinfo_write( imageinfo, filename ) )
-		return( FALSE );
-	if( !(im = im_open( filename, "r" )) ) {
-		error_vips_all();
-		return( FALSE );
-	}
-
-	IM_FREEF( im_close, imageinfo->im );
-	IM_FREEF( im_close, imageinfo->mapped_im );
-	MANAGED_UNREF( imageinfo->underlying );
-
-	imageinfo->im = im;
-
-	/* It's now a temp file.
-	 */
-	imageinfo->dfile = TRUE;
-
-	iobject_changed( IOBJECT( imageinfo ) );
-
-	return( TRUE );
-}
-
 static gboolean
 imageinfo_make_paintable( Imageinfo *imageinfo )
-{
-	if( !imageinfo_file( imageinfo ) ) 
-		return( FALSE );
+{	
+	progress_begin();
 	if( im_rwcheck( imageinfo->im ) ) {
+		progress_end();
 		error_top( _( "Unable to paint on image." ) );
 		error_sub( _( "Unable to get write permission for "
 			"file \"%s\".\nCheck permission settings." ), 
@@ -1180,6 +1136,7 @@ imageinfo_make_paintable( Imageinfo *imageinfo )
 		error_vips();
 		return( FALSE );
 	}
+	progress_end();
 
 	imageinfo->ok_to_paint = TRUE;
 
@@ -1255,120 +1212,6 @@ imageinfo_sym_image( Symbol *sym )
                 return( NULL );
 }
 
-/* Brush definitions.
- */
-static PEL imageinfo_brush1[] = {
-	0xff
-};
-static PEL imageinfo_brush2[] = {
-	0xff, 0xff,
-	0xff, 0xff
-};
-static PEL imageinfo_brush3[] = {
-	0x00, 0xff, 0x00,
-	0xff, 0xff, 0xff,
-	0x00, 0xff, 0x00
-};
-static PEL imageinfo_brush4[] = {
-	0x00, 0xff, 0xff, 0x00,
-	0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff,
-	0x00, 0xff, 0xff, 0x00
-};
-static PEL imageinfo_brush5[] = {
-	0x00, 0xff, 0xff, 0xff, 0x00,
-	0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff,
-	0x00, 0xff, 0xff, 0xff, 0x00
-};
-static PEL imageinfo_brush6[] = {
-	0x00, 0x00, 0xff, 0xff, 0x00, 0x00,
-	0x00, 0xff, 0xff, 0xff, 0xff, 0x00,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0x00, 0xff, 0xff, 0xff, 0xff, 0x00,
-	0x00, 0x00, 0xff, 0xff, 0x00, 0x00,
-};
-static PEL imageinfo_brush7[] = {
-	0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
-	0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
-	0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
-	0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-	0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
-	0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
-	0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
-	0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
-};
-static PEL imageinfo_brush8[] = {
-	0x00, 0xff,
-	0xff, 0xff
-};
-static PEL imageinfo_brush9[] = {
-	0x00, 0x00, 0xff,
-	0x00, 0xff, 0xff,
-	0xff, 0xff, 0x00
-};
-static PEL imageinfo_brush10[] = {
-	0x00, 0x00, 0x00, 0xff,
-	0x00, 0x00, 0xff, 0xff,
-	0x00, 0xff, 0xff, 0x00,
-	0xff, 0xff, 0x00, 0x00
-};
-static PEL imageinfo_brush11[] = {
-	0x00, 0x00, 0x00, 0x00, 0xff,
-	0x00, 0x00, 0x00, 0xff, 0xff,
-	0x00, 0x00, 0xff, 0xff, 0x00,
-	0x00, 0xff, 0xff, 0x00, 0x00,
-	0xff, 0xff, 0x00, 0x00, 0x00
-};
-static PEL imageinfo_brush12[] = {
-	0x00, 0x00, 0x00, 0x00, 0x00, 0xff,
-	0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
-	0x00, 0x00, 0x00, 0xff, 0xff, 0x00,
-	0x00, 0x00, 0xff, 0xff, 0x00, 0x00,
-	0x00, 0xff, 0xff, 0x00, 0x00, 0x00,
-	0xff, 0xff, 0x00, 0x00, 0x00, 0x00
-};
-static PEL imageinfo_brush13[] = {
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff,
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00,
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00,
-	0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00,
-	0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
-	0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
-	0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-	0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-	0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
-};
-
-static PEL *imageinfo_brush_masks[] = {
-	imageinfo_brush1, imageinfo_brush2, imageinfo_brush3, 
-	imageinfo_brush4, imageinfo_brush5, imageinfo_brush6, 
-	imageinfo_brush7, imageinfo_brush8, imageinfo_brush9, 
-	imageinfo_brush10, imageinfo_brush11, imageinfo_brush12, 
-	imageinfo_brush13
-};
-
-static Rect imageinfo_brush_shapes[] = {
-	{ 0, 0, 1, 1 },			/* PAINTBOX_1ROUND */
-	{ -1, -1, 2, 2 },		/* PAINTBOX_2ROUND */
-	{ -1, -1, 3, 3 },		/* PAINTBOX_3ROUND */
-	{ -2, -2, 4, 4 },		/* PAINTBOX_4ROUND */
-	{ -2, -2, 5, 5 },		/* PAINTBOX_5ROUND */
-	{ -3, -3, 6, 6 },		/* PAINTBOX_6ROUND */
-	{ -5, -5, 10, 10 },		/* PAINTBOX_10ROUND */
-	{ -1, -1, 2, 2 },		/* PAINTBOX_2ITALIC */
-	{ -1, -1, 3, 3 },		/* PAINTBOX_3ITALIC */
-	{ -2, -2, 4, 4 },		/* PAINTBOX_4ITALIC */
-	{ -2, -2, 5, 5 },		/* PAINTBOX_5ITALIC */
-	{ -3, -3, 6, 6 },		/* PAINTBOX_6ITALIC */
-	{ -5, -5, 10, 10 }		/* PAINTBOX_10ITALIC */
-};
-
 static Undofragment *
 imageinfo_undofragment_new( Undobuffer *undo )
 {
@@ -1744,9 +1587,6 @@ imageinfo_redo( Imageinfo *imageinfo )
 	return( TRUE );
 }
 
-/* Try to add to the undo buffer. If undo is disabled, junk everything since
- * this new paint action will invalidate all saved undo/redo stuff.
- */
 void
 imageinfo_undo_clear( Imageinfo *imageinfo )
 {
@@ -1754,23 +1594,37 @@ imageinfo_undo_clear( Imageinfo *imageinfo )
 	imageinfo_undo_changed( imageinfo );
 }
 
+static int
+imageinfo_draw_point_cb( IMAGE *im, int x, int y, void *a, void *b, void *c )
+{
+	IMAGE *mask = (IMAGE *) a;
+	PEL *ink = (PEL *) b;
+
+	return( im_draw_mask( im, mask, 
+		x - mask->Xsize / 2, y - mask->Ysize / 2, ink ) );
+}
+
 /* Draw a line.
  */
 gboolean
-imageinfo_paint_line( Imageinfo *imageinfo, Imageinfo *ink, 
-	int nib, int x1, int y1, int x2, int y2 )
+imageinfo_paint_line( Imageinfo *imageinfo, 
+	Imageinfo *ink, Imageinfo *mask,
+	int x1, int y1, int x2, int y2 )
 {
 	IMAGE *im = imageinfo_get( FALSE, imageinfo ); 
 	IMAGE *ink_im = imageinfo_get( FALSE, ink );
+	IMAGE *mask_im = imageinfo_get( FALSE, mask );
 	PEL *data = (PEL *) ink_im->data;
 	Rect dirty, p1, p2, image, clipped;
 
-	p1 = imageinfo_brush_shapes[nib];
-	p1.left += x1;
-	p1.top += y1;
-	p2 = imageinfo_brush_shapes[nib];
-	p2.left += x2;
-	p2.top += y2;
+	p1.width = mask_im->Xsize;
+	p1.height = mask_im->Ysize;
+	p1.left = x1 - mask_im->Xsize / 2;
+	p1.top = y1 - mask_im->Ysize / 2;
+	p2.width = mask_im->Xsize;
+	p2.height = mask_im->Ysize;
+	p2.left = x2 - mask_im->Xsize / 2;
+	p2.top = y2 - mask_im->Ysize / 2;
 	im_rect_unionrect( &p1, &p2, &dirty );
 
 	image.left = 0;
@@ -1785,8 +1639,8 @@ imageinfo_paint_line( Imageinfo *imageinfo, Imageinfo *ink,
 	if( !imageinfo_undo_add( imageinfo, &clipped ) ) 
 		return( FALSE );
 
-	if( im_fastlineuser( im, x1, y1, x2, y2, im_plotmask, data, 
-		imageinfo_brush_masks[nib], &imageinfo_brush_shapes[nib] ) ) {
+	if( im_draw_line_user( im, x1, y1, x2, y2, 
+		(VipsPlotFn) imageinfo_draw_point_cb, mask_im, data, NULL ) ) {
 		error_vips_all();
 		return( FALSE );
 	}
@@ -1819,8 +1673,8 @@ imageinfo_paint_smudge( Imageinfo *imageinfo,
 
 	/* Smudge line connecting old and new points. 
 	 */
-	if( im_fastlineuser( im, x1, y1, x2, y2, im_smudge, 
-		oper, NULL, NULL ) ) {
+	if( im_draw_line_user( im, x1, y1, x2, y2, 
+		(VipsPlotFn) im_smudge, oper, NULL, NULL ) ) {
 		error_vips_all();
 		return( FALSE );
 	}
@@ -1842,7 +1696,7 @@ imageinfo_paint_flood( Imageinfo *imageinfo, Imageinfo *ink,
 	Rect dirty;
 	int result;
 
-	/* Save undo area. We have to save the entire image, as we don't know
+	/* Save undo area. We have to save the entire image since we don't know
 	 * how much the flood will change :(
 	 */
 	dirty.left = 0;
@@ -1904,7 +1758,8 @@ imageinfo_paint_rect( Imageinfo *imageinfo, Imageinfo *ink, Rect *area )
 	if( !imageinfo_undo_add( imageinfo, area ) )
 		return( FALSE );
 
-	if( im_paintrect( im, area, data ) ) {
+	if( im_draw_rect( im, 
+		area->left, area->top, area->width, area->height, 1, data ) ) {
 		error_vips_all();
 		return( FALSE );
 	}
@@ -1939,6 +1794,41 @@ imageinfo_paint_text( Imageinfo *imageinfo,
 	return( TRUE );
 }
 
+/* Draw a nib mask. Radius 0 means a single-pixel mask.
+ */
+gboolean
+imageinfo_paint_nib( Imageinfo *imageinfo, int radius )
+{
+	static PEL ink[1] = { 255 };
+
+	IMAGE *im = imageinfo_get( FALSE, imageinfo );
+
+	if( radius ) {
+		int r2 = radius * 2;
+		IMAGE *t;
+
+		if( !(t = im_open( "imageinfo_paint_nib", "p" )) ) {
+			error_vips();
+			return( FALSE );
+		}
+		if( im_black( t, 2 * (r2 + 1), 2 * (r2 + 1), 1 ) ||
+			im_draw_circle( t, r2, r2, r2, 1, ink ) ||
+			im_shrink( t, im, 2, 2 ) ) {
+			im_close( t );
+			error_vips();
+			return( FALSE );
+		}
+		im_close( t );
+	}
+	else {
+		if( im_black( im, 1, 1, 1 ) ||
+			im_draw_circle( im, 0, 0, 0, 1, ink ) )
+			return( FALSE );
+	}
+
+	return( TRUE );
+}
+
 /* Paint a mask.
  */
 gboolean
