@@ -449,6 +449,8 @@ imagepresent_floating_new( Imagepresent *ip,
 	gboolean frozen, RegionviewType type, RegionviewResize resize,
 	int x, int y )
 {
+	g_assert( !ip->regionview );
+
 	ip->floating.left = left;
 	ip->floating.top = top;
 	ip->floating.width = width;
@@ -801,9 +803,16 @@ imagepresent_left_press( Imagepresent *ip, GdkEvent *ev, int x, int y )
 	IMAGE *im, *im2;
 	int ix, iy;
 
+	/* If there's a regionview grabbed already, block other actions. This
+	 * can happen with, for example, the win32 backend where we don't
+	 * always see a RELEASE for every PRESS.
+	 */
+	if( ip->regionview )
+		return( FALSE );
+
 	switch( imagemodel->state ) {
 	case IMAGEMODEL_SELECT:
-		if( !ip->regionview && ev->button.state & GDK_CONTROL_MASK ) {
+		if( ev->button.state & GDK_CONTROL_MASK ) {
 			im = imageinfo_get( FALSE, conv->ii );
 
 			imagepresent_snap_point( ip, x, y, &x, &y );
@@ -843,6 +852,7 @@ imagepresent_left_press( Imagepresent *ip, GdkEvent *ev, int x, int y )
 
 	case IMAGEMODEL_PEN:
 	case IMAGEMODEL_SMUDGE:
+		imagemodel_refresh_nib( imagemodel );
 		imagepresent_snap_point( ip, x, y, &x, &y );
 		conversion_disp_to_im( conv, x, y, &ix, &iy );
 		ip->paint_last_x = ix;
@@ -851,6 +861,7 @@ imagepresent_left_press( Imagepresent *ip, GdkEvent *ev, int x, int y )
 		break;
 
 	case IMAGEMODEL_LINE:
+		imagemodel_refresh_nib( imagemodel );
 		imagepresent_snap_point( ip, x, y, &x, &y );
 		conversion_disp_to_im( conv, x, y, &ix, &iy );
 		ip->paint_last_x = ix;
@@ -910,7 +921,7 @@ imagepresent_paint_stop( Imagepresent *ip, int x, int y )
 {
 	Imagemodel *imagemodel = ip->imagemodel;
 	Conversion *conv = imagemodel->conv;
-	Imageinfo *imageinfo = imagemodel->iimage->value.ii;
+	Imageinfo *imageinfo = conv->ii;
         Rect oper;
         int ix, iy;
 
@@ -929,7 +940,8 @@ imagepresent_paint_stop( Imagepresent *ip, int x, int y )
 
 	case IMAGEMODEL_PEN:
 		if( !imageinfo_paint_line( imageinfo, 
-			imagemodel->ink, imagemodel->nib, 
+			imagemodel->ink, 
+			imagemodel->nib, 
 			ip->paint_last_x, ip->paint_last_y, ix, iy ) )
 			iwindow_alert( GTK_WIDGET( ip ), GTK_MESSAGE_ERROR );
 
@@ -940,7 +952,8 @@ imagepresent_paint_stop( Imagepresent *ip, int x, int y )
 			DESTROY_GTK( ip->regionview );
 
 			if( !imageinfo_paint_line( imageinfo, 
-				imagemodel->ink, imagemodel->nib, 
+				imagemodel->ink, 
+				imagemodel->nib, 
 				ip->floating.left, ip->floating.top,
 				IM_RECT_RIGHT( &ip->floating ),
 				IM_RECT_BOTTOM( &ip->floating ) ) )
@@ -1015,6 +1028,10 @@ imagepresent_paint_stop( Imagepresent *ip, int x, int y )
 
 	imagemodel_paint_recalc( imagemodel );
 	imageinfo_undo_mark( imageinfo );
+
+	/* Ask everyone to drop cache, the image has changed.
+	 */
+	im_invalidate( imageinfo_get( FALSE, imageinfo ) );
 }
 
 /* Left button release event.
@@ -1023,7 +1040,8 @@ static void
 imagepresent_left_release( Imagepresent *ip, GdkEvent *ev, int x, int y )
 {
 	Imagemodel *imagemodel = ip->imagemodel;
-	Row *row = HEAPMODEL( imagemodel->iimage )->row;
+	Row *row = imagemodel->iimage ? 
+		HEAPMODEL( imagemodel->iimage )->row : NULL;
 
 	switch( imagemodel->state ) {
 	case IMAGEMODEL_SELECT:
@@ -1129,7 +1147,7 @@ imagepresent_button_motion( Imagepresent *ip, GdkEvent *ev )
 {
 	Imagemodel *imagemodel = ip->imagemodel;
 	Conversion *conv = imagemodel->conv;
-	Imageinfo *imageinfo = imagemodel->iimage->value.ii;
+	Imageinfo *imageinfo = conv->ii;
 	Rect oper;
 	int x, y;
 	int ix, iy;
@@ -1163,9 +1181,12 @@ imagepresent_button_motion( Imagepresent *ip, GdkEvent *ev )
 
 	case IMAGEMODEL_PEN:
 		if( !imageinfo_paint_line( imageinfo, 
-			imagemodel->ink, imagemodel->nib, 
+			imagemodel->ink, 
+			imagemodel->nib, 
 			ip->paint_last_x, ip->paint_last_y, ix, iy ) )
 			iwindow_alert( GTK_WIDGET( ip ), GTK_MESSAGE_ERROR );
+		im_invalidate( imageinfo_get( FALSE, imageinfo ) );
+
 		ip->paint_last_x = ix;
 		ip->paint_last_y = iy;
 		break;
@@ -1191,13 +1212,17 @@ imagepresent_button_motion( Imagepresent *ip, GdkEvent *ev )
 			ip->paint_last_x, ip->paint_last_y, 
 			ix, iy ) )
 			iwindow_alert( GTK_WIDGET( ip ), GTK_MESSAGE_ERROR );
+		im_invalidate( imageinfo_get( FALSE, imageinfo ) );
+
 		ip->paint_last_x = ix;
 		ip->paint_last_y = iy;
+
 		break;
 
 	default:
 		break;
 	}
+
 }
 
 /* Main event loop.
@@ -1673,8 +1698,10 @@ imagepresent_link( Imagepresent *ip, Imagemodel *imagemodel )
 		G_CALLBACK( imagepresent_imagemodel_imageinfo_changed_cb ), 
 		ip );
 	imagedisplay_set_conversion( ip->id, imagemodel->conv );
-	imagemodel->iimage->views = 
-		g_slist_prepend( imagemodel->iimage->views, ip );
+
+	if( imagemodel->iimage )
+		imagemodel->iimage->views = 
+			g_slist_prepend( imagemodel->iimage->views, ip );
 }
 
 /* Make a new Imagepresent. 
