@@ -259,7 +259,7 @@ main_print_ws( Workspace *ws, gboolean *found )
 }
 
 /* Clean up our application and quit. Not interactive! Do any "has been
- * modified, OK to quit?" stuff before this.
+ * modified, OK to quit?" stuff before this, see main_quit_test().
  */
 static void
 main_quit( void )
@@ -293,23 +293,21 @@ main_quit( void )
 			main_error_exit( "%s", _( "no \"main\" found" ) );
 	}
 
-	/* Force all windows down. 
-
-		FIXME 
-
-		this can cause a recursive call to us from mainw_destroy()
-
+	/* Force all our windows down.
 	 */
 	iwindow_map_all( (iWindowMapFn) iwindow_kill, NULL );
+
+	/* Saves recent and stuff like that.
+	 */
 	mainw_shutdown();
 
 	/* Dump wisdom back again.
 	 */
 #if HAVE_FFTW || HAVE_FFTW3
-	if( (of = file_open_write( "%s" G_DIR_SEPARATOR_S "wisdom", 
+	if( (of = ifile_open_write( "%s" G_DIR_SEPARATOR_S "wisdom", 
 		get_savedir() )) ) {
 		fftw_export_wisdom_to_file( of->fp );
-		file_close( of );
+		ifile_close( of );
 	}
 #endif /*HAVE_FFTW*/
 
@@ -336,6 +334,7 @@ main_quit( void )
 	 */
 	if( main_icon_factory )
 		gtk_icon_factory_remove_default( main_icon_factory );
+	junk_tooltips();
 
 #ifdef HAVE_LIBGOFFICE
 	/* Not quite sure what this does, but don't do it in batch mode.
@@ -373,20 +372,44 @@ main_quit( void )
 	exit( 0 );
 }
 
+/* We mustn't quit recursively!
+ */
+static gboolean main_quit_running = FALSE;
+
 static void
 main_quit_test_cb( void *sys, iWindowResult result )
 {
+#ifdef DEBUG
+	printf( "main_quit_test_cb:\n" );
+#endif/*DEBUG*/
+
 	if( result == IWINDOW_YES )
 		/* No return from this.
 		 */
 		main_quit();
+	else
+		/* Quit has been cancelled.
+		 */
+		main_quit_running = FALSE;
 }
 
-/* Check before quitting.
+/* Check before quitting. 
  */
 void
 main_quit_test( void )
 {
+	if( main_quit_running ) {
+#ifdef DEBUG
+		printf( "main_quit_test: recursive quit blocked\n" );
+#endif/*DEBUG*/
+		return;
+	}
+	main_quit_running = TRUE;
+
+#ifdef DEBUG
+	printf( "main_quit_test:\n" );
+#endif/*DEBUG*/
+
 	/* Flush any pending preference saves before we look for dirty
 	 * objects.
 	 */
@@ -1000,10 +1023,6 @@ main( int argc, char *argv[] )
  	 */
 	gtk_disable_setlocale();
 
-	/* Set localised application name.
-	 */
-	g_set_application_name( _( PACKAGE ) );
-
 #ifdef DEBUG
 	printf( "main: sizeof( HeapNode ) == %zd\n", sizeof( HeapNode ) );
 
@@ -1076,6 +1095,10 @@ main( int argc, char *argv[] )
 	bindtextdomain( GETTEXT_PACKAGE, name );
 	bind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );
 
+	/* Set localised application name.
+	 */
+	g_set_application_name( _( PACKAGE ) );
+
 	context = g_option_context_new( _( "- image processing spreadsheet" ) );
 	g_option_context_add_main_entries( context, 
 		main_option, GETTEXT_PACKAGE );
@@ -1148,7 +1171,7 @@ main( int argc, char *argv[] )
 #endif /*OS_WIN32*/ 
 #endif /*DEBUG_FATAL*/
 
-	main_stdin = file_open_read_stdin();
+	main_stdin = ifile_open_read_stdin();
 
 #ifdef HAVE_GETRLIMIT
 	/* Make sure we have lots of file descriptors. Some platforms have cur
@@ -1204,10 +1227,10 @@ main( int argc, char *argv[] )
 	fftw_import_system_wisdom();
 #endif /*HAVE_FFTW3*/
 #if HAVE_FFTW || HAVE_FFTW3
-	if( (of = file_open_read( "%s" G_DIR_SEPARATOR_S "wisdom", 
+	if( (of = ifile_open_read( "%s" G_DIR_SEPARATOR_S "wisdom", 
 		get_savedir() )) ) {
 		fftw_import_wisdom_from_file( of->fp );
-		file_close( of );
+		ifile_close( of );
 	}
 #endif /*HAVE_FFTW*/
 
@@ -1466,11 +1489,11 @@ main( int argc, char *argv[] )
 			nativeize_path( save_dir );
 			error_top( "%s", buf );
 			error_sub( 
-_( "A new directory has been created in your home directory to hold startup, "
+_( "A new directory has been created to hold startup, "
 "data and temporary files:\n\n"
 "     %s\n\n"
-"If you've used previous versions of %s, you will probably want "
-"to move any files over from your old work area and remove any old temps." ),
+"If you've used previous versions of %s, you might want "
+"to copy files over from your old work area." ),
 				save_dir, PACKAGE );
 			iwindow_alert( NULL, GTK_MESSAGE_INFO );
 		}
