@@ -143,10 +143,14 @@ mainw_destroy( GtkObject *object )
 
 	/* My instance destroy stuff.
 	 */
-	if( mainw->ws )
+	if( mainw->ws ) {
 		mainw->ws->iwnd = NULL;
-	FREESID( mainw->destroy_sid, mainw->ws );
-	FREESID( mainw->changed_sid, mainw->ws );
+
+		filemodel_set_window_hint( FILEMODEL( mainw->ws ), NULL );
+		FREESID( mainw->destroy_sid, mainw->ws );
+		FREESID( mainw->changed_sid, mainw->ws );
+	}
+
 	FREESID( mainw->imageinfo_changed_sid, main_imageinfogroup );
 	FREESID( mainw->heap_changed_sid, reduce_context->heap );
 	FREESID( mainw->watch_changed_sid, main_watchgroup );
@@ -326,6 +330,10 @@ mainw_cancel_cb( GtkWidget *wid, Mainw *mainw )
 static void
 mainw_workspace_destroy_cb( Workspace *ws, Mainw *mainw )
 {
+#ifdef DEBUG
+	printf( "mainw_workspace_destroy_cb: %p\n", mainw );
+#endif /*DEBUG*/
+
 	mainw->destroy_sid = 0;
 	mainw->changed_sid = 0;
 	mainw->ws = NULL;
@@ -409,7 +417,8 @@ mainw_title_update( Mainw *mainw )
 			mainw->ws->compat_minor ); 
 	}
 	if( FILEMODEL( mainw->ws )->filename )
-		vips_buf_appendf( &buf, " - %s", FILEMODEL( mainw->ws )->filename );
+		vips_buf_appendf( &buf, " - %s", 
+			FILEMODEL( mainw->ws )->filename );
 	else {
 		vips_buf_appends( &buf, " - " );
 		vips_buf_appends( &buf, _( "unsaved workspace" ) );
@@ -428,9 +437,7 @@ mainw_status_update( Mainw *mainw )
 	else {
 		char txt[256];
 
-		im_snprintf( txt, 256, 
-			_( "%s: &#0169;2009 Imperial College, London" ), 
-			PACKAGE );
+		im_snprintf( txt, 256, _( NIP_COPYRIGHT ), PACKAGE );
 		gtk_label_set_markup( GTK_LABEL( mainw->statusbar ), txt );
 	}
 }
@@ -513,8 +520,10 @@ mainw_refresh( Mainw *mainw )
 		"NoEdit"
 	};
 
+	iWindow *iwnd = IWINDOW( mainw );
 	Workspace *ws = mainw->ws;
 	int pref = IM_CLIP( 0, MAINW_TOOLBAR_STYLE, IM_NUMBER( styles ) - 1 );
+
         GtkAction *action;
 
 #ifdef DEBUG
@@ -531,34 +540,34 @@ mainw_refresh( Mainw *mainw )
 
 	gtk_toolbar_set_style( GTK_TOOLBAR( mainw->toolbar ), styles[pref] );
 
-	action = gtk_action_group_get_action( mainw->action_group, 
+	action = gtk_action_group_get_action( iwnd->action_group, 
 		"AutoRecalculate" );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
 		mainw_auto_recalc );
 
-	action = gtk_action_group_get_action( mainw->action_group, 
+	action = gtk_action_group_get_action( iwnd->action_group, 
 		"Toolbar" );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
 		mainw->toolbar_visible );
         widget_visible( mainw->toolbar, mainw->toolbar_visible );
 
-	action = gtk_action_group_get_action( mainw->action_group, 
+	action = gtk_action_group_get_action( iwnd->action_group, 
 		"Statusbar" );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
 		mainw->statusbar_visible );
         widget_visible( mainw->statusbar_main, mainw->statusbar_visible );
 
-	action = gtk_action_group_get_action( mainw->action_group, 
+	action = gtk_action_group_get_action( iwnd->action_group, 
 		"WorkspaceDefs" );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
 		mainw->lpane->open );
 
-	action = gtk_action_group_get_action( mainw->action_group, 
+	action = gtk_action_group_get_action( iwnd->action_group, 
 		"ToolkitBrowser" );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
 		mainw->rpane->open );
 
-	action = gtk_action_group_get_action( mainw->action_group, 
+	action = gtk_action_group_get_action( iwnd->action_group, 
 		view_mode[ws->mode] );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
 		TRUE );
@@ -608,6 +617,9 @@ mainw_space_free_tooltip_generate( GtkWidget *widget, VipsBuf *buf,
         vips_buf_appends( buf, ", " );
 
         vips_buf_appendf( buf, _( "%d vips calls cached" ), vips_history_size );
+        vips_buf_appends( buf, ", " );
+
+        vips_buf_appendf( buf, _( "using %d threads" ), im_concurrency_get() );
 }
 
 static void
@@ -619,10 +631,10 @@ mainw_free_changed_cb( gpointer *dummy, Mainw *mainw )
 
 /* Go to home page.
  */
-static void
-mainw_homepage_action_cb( GtkAction *action, Mainw *mainw )
+void
+mainw_homepage_action_cb( GtkAction *action, iWindow *iwnd )
 {
-	box_url( GTK_WIDGET( mainw ), VIPS_HOMEPAGE );
+	box_url( GTK_WIDGET( iwnd ), VIPS_HOMEPAGE );
 }
 
 /* About... box.
@@ -879,44 +891,18 @@ mainw_workspace_save_as_action_cb( GtkAction *action, Mainw *mainw )
 	filemodel_inter_saveas( IWINDOW( mainw ), FILEMODEL( mainw->ws ) );
 }
 
-Workspace *
-mainw_open_file_into_workspace( Mainw *mainw, const char *filename )
+static Workspace *
+mainw_open_workspace( Workspacegroup *wsg, const char *filename )
 {
-	Workspacegroup *wsg = WORKSPACEGROUP( ICONTAINER( mainw->ws )->parent );
-	Workspace *new_ws;
-	Mainw *new_mainw;
+	Workspace *ws;
+	Mainw *mainw;
 
-	if( !(new_ws = workspace_new_from_file( wsg, filename )) ) 
+	if( !(ws = workspace_new_from_file( wsg, filename )) ) 
 		return( NULL );
 
-	new_mainw = mainw_new( new_ws );
-	gtk_widget_show( GTK_WIDGET( new_mainw ) );
-
-	/*
-
-		FIXME ... if the ws we are loading into is blank, we could
-		call workspace_merge_file() instead of
-		workspace_new_from_file() and avoid the uglyliness. But this
-		will fail if there's a version mismatch and filename needs to
-		have compatibility menus.
-
-		Could open filename and get version info, then decide whether
-		we need a new window of whether we load into this window.
-
-	*/
-	if( workspace_is_empty( mainw->ws ) ) {
-		/* Make sure modified isn't set ... otherwise we'll get a
-		 * "save before close" dialog.
-		 */
-		filemodel_set_modified( FILEMODEL( mainw->ws ), FALSE );
-		iwindow_kill( IWINDOW( mainw ) );
-	}
-
-	/* Can't rely on filename still being around ... read filename
-	 * from new ws instead.
-	 */
-	mainw_recent_add( &mainw_recent_workspace, 
-		FILEMODEL( new_ws )->filename );
+	mainw = mainw_new( ws );
+	gtk_widget_show( GTK_WIDGET( mainw ) );
+	mainw_recent_add( &mainw_recent_workspace, filename );
 
 	/* Process some events to make sure we rethink the layout and
 	 * are able to get the append-at-RHS offsets.
@@ -926,13 +912,42 @@ mainw_open_file_into_workspace( Mainw *mainw, const char *filename )
 
 	symbol_recalculate_all();
 
+	return( ws );
+}
+
+/* Open a new workspace, close the current one if it's empty. 
+ *
+ * The idea is that if you have a blank workspace and do file-open, you want
+ * to load into the blank. However, we can't actually do this, since the new
+ * workspace might need a different set of menus. 
+ *
+ * We could sniff the file and then call workspace_merge_file() if the
+ * workspace was compatible with the current one, but why bother.
+ */
+Workspace *
+mainw_open_file_into_workspace( Mainw *mainw, const char *filename )
+{
+	Workspacegroup *wsg = WORKSPACEGROUP( ICONTAINER( mainw->ws )->parent );
+	Workspace *new_ws;
+
+	if( !(new_ws = mainw_open_workspace( wsg, filename )) ) 
+		return( NULL );
+
+	if( workspace_is_empty( mainw->ws ) ) {
+		/* Make sure modified isn't set ... otherwise we'll get a
+		 * "save before close" dialog.
+		 */
+		filemodel_set_modified( FILEMODEL( mainw->ws ), FALSE );
+		iwindow_kill( IWINDOW( mainw ) );
+	}
+
 	return( new_ws );
 }
 
 /* Track these during a load.
  */
 typedef struct {
-	Mainw *mainw;
+	Workspacegroup *wsg;
 	VipsBuf *buf;
 	int nitems;
 } MainwLoad;
@@ -944,7 +959,7 @@ static void *
 mainw_open_fn( Filesel *filesel, const char *filename, MainwLoad *load )
 {
 	if( is_file_type( &filesel_wfile_type, filename ) ) {
-		if( !mainw_open_file_into_workspace( load->mainw, filename ) )
+		if( !mainw_open_workspace( load->wsg, filename ) )
 			return( filesel );
 	}
 	else {
@@ -967,37 +982,53 @@ mainw_open_done_cb( iWindow *iwnd, void *client,
 {
 	Mainw *mainw = MAINW( client );
 	Filesel *filesel = FILESEL( iwnd );
-	int nselected = filesel_nselected( filesel );
 	char txt[MAX_STRSIZE];
 	VipsBuf buf = VIPS_BUF_STATIC( txt );
 	MainwLoad load;
 
-	load.mainw = mainw;
+	load.wsg = WORKSPACEGROUP( ICONTAINER( mainw->ws )->parent );
 	load.buf = &buf;
 	load.nitems = 0;
 
-	if( nselected > 1 )
-		vips_buf_appends( &buf, "Group [" );
 	if( filesel_map_filename_multi( filesel,
 		(FileselMapFn) mainw_open_fn, &load, NULL ) ) {
 		nfn( sys, IWINDOW_ERROR );
 		return;
 	}
-	if( nselected > 1 )
-		vips_buf_appends( &buf, "]" );
 
-	/* We may have loaded a workspace and caused this workspace to
-	 * vanish, argh. mainw may no longer be valid, but check ->ws anyway.
-	 * We'll probably see a NULL if this ws has been junked.
+	/* If there's nothing left to load (we only had workspaes to load and
+	 * we've loaded them all) and the current workspace is smpty, we can 
+	 * junk it.
 	 */
-	if( load.nitems && 
-		mainw->ws &&
-		!workspace_add_def( mainw->ws, vips_buf_all( &buf ) ) ) {
-		error_top( _( "Load failed." ) );
-		error_sub( _( "Unable to execute:\n   %s" ), 
-			vips_buf_all( &buf ) );
-		nfn( sys, IWINDOW_ERROR );
-		return;
+	if( !load.nitems && 
+		workspace_is_empty( mainw->ws ) ) {
+		/* Make sure modified isn't set ... otherwise we'll get a
+		 * "save before close" dialog.
+		 */
+		filemodel_set_modified( FILEMODEL( mainw->ws ), FALSE );
+		iwindow_kill( IWINDOW( mainw ) );
+	}
+
+	/* Some actual files (image, matrix) were selected. Load into
+	 * the current workspace.
+	 */
+	if( load.nitems ) {
+		char txt2[MAX_STRSIZE];
+		VipsBuf buf2 = VIPS_BUF_STATIC( txt2 );
+
+		if( load.nitems > 1 )
+			vips_buf_appendf( &buf2, "Group [%s]", 
+				vips_buf_all( &buf ) );
+		else
+			vips_buf_appends( &buf2, vips_buf_all( &buf ) );
+
+		if( !workspace_add_def( mainw->ws, vips_buf_all( &buf2 ) ) ) {
+			error_top( _( "Load failed." ) );
+			error_sub( _( "Unable to execute:\n   %s" ), 
+				vips_buf_all( &buf2 ) );
+			nfn( sys, IWINDOW_ERROR );
+			return;
+		}
 	}
 
 	nfn( sys, IWINDOW_YES );
@@ -1268,16 +1299,22 @@ mainw_workspace_duplicate_action_cb( GtkAction *action, Mainw *mainw )
 	Mainw *new_mainw;
 
         progress_begin();
+
 	if( !(new_ws = workspace_clone( mainw->ws )) ) {
 		progress_end();
 		iwindow_alert( GTK_WIDGET( mainw ), GTK_MESSAGE_ERROR );
 		return;
 	}
-	symbol_recalculate_all();
-	progress_end();
 
 	new_mainw = mainw_new( new_ws );
 	gtk_widget_show( GTK_WIDGET( new_mainw ) );
+
+	/* We have to show before recalculate or the window never appears, not
+	 * sure why.
+	 */
+	symbol_recalculate_all();
+
+	progress_end();
 }
 
 static void
@@ -1456,7 +1493,10 @@ mainw_statusbar_action_cb( GtkToggleAction *action, Mainw *mainw )
 static void
 mainw_toolkitbrowser_action_cb( GtkToggleAction *action, Mainw *mainw )
 {
-	pane_set_open( mainw->rpane, gtk_toggle_action_get_active( action ) );
+	if( gtk_toggle_action_get_active( action ) )
+		pane_animate_open( mainw->rpane );
+	else
+		pane_animate_closed( mainw->rpane );
 }
 
 /* Expose/hide the workspace defs.
@@ -1464,7 +1504,10 @@ mainw_toolkitbrowser_action_cb( GtkToggleAction *action, Mainw *mainw )
 static void
 mainw_workspacedefs_action_cb( GtkToggleAction *action, Mainw *mainw )
 {
-	pane_set_open( mainw->lpane, gtk_toggle_action_get_active( action ) );
+	if( gtk_toggle_action_get_active( action ) )
+		pane_animate_open( mainw->lpane );
+	else
+		pane_animate_closed( mainw->lpane );
 }
 
 /* Layout columns.
@@ -1573,6 +1616,17 @@ mainw_magic_cb( gpointer callback_data, guint callback_action,
 }
  */
 
+#ifdef HAVE_LIBGVC
+static void
+mainw_graph_action_cb( GtkAction *action, Mainw *mainw )
+{
+	Graphwindow *graphwindow;
+
+	graphwindow = graphwindow_new( mainw->ws, GTK_WIDGET( mainw ) );
+	gtk_widget_show( GTK_WIDGET( graphwindow ) );
+}
+#endif /*HAVE_LIBGVC*/
+
 /* Set display mode.
  */
 static void
@@ -1588,14 +1642,9 @@ mainw_mode_action_cb( GtkRadioAction *action, GtkRadioAction *current,
 static GtkActionEntry mainw_actions[] = {
 	/* Menu items.
 	 */
-	{ "FileMenu", NULL, N_( "_File" ) },
-	{ "NewMenu", NULL, N_( "_New" ) },
 	{ "RecentMenu", NULL, N_( "Open _Recent" ) },
-	{ "EditMenu", NULL, N_( "_Edit" ) },
 	{ "JumpToColumnMenu", NULL, N_( "Jump to _Column" ) },
-	{ "ViewMenu", NULL, N_( "_View" ) },
 	{ "ToolkitsMenu", NULL, N_( "_Toolkits" ) },
-	{ "HelpMenu", NULL, N_( "_Help" ) },
 
 	/* Dummy action ... replaced at runtime.
 	 */
@@ -1654,11 +1703,6 @@ static GtkActionEntry mainw_actions[] = {
 		N_( "Load last automatically backed-up workspace" ), 
 		G_CALLBACK( mainw_recover_action_cb ) },
 
-	{ "Close", 
-		GTK_STOCK_CLOSE, N_( "_Close" ), "<control>w",
-		N_( "Close workspace" ), 
-		G_CALLBACK( iwindow_kill_action_cb ) },
-
 	{ "Delete", 
 		GTK_STOCK_DELETE, N_( "_Delete" ), "<Shift>BackSpace",
 		N_( "Delete selected items" ), 
@@ -1714,25 +1758,17 @@ static GtkActionEntry mainw_actions[] = {
 		N_( "Edit preferences" ), 
 		G_CALLBACK( mainw_preferences_action_cb ) },
 
+#ifdef HAVE_LIBGVC
+	{ "Graph", 
+		NULL, N_( "Workspace as Grap_h" ), NULL,
+		N_( "Show a graph of workspace dependencies" ), 
+		G_CALLBACK( mainw_graph_action_cb ) },
+#endif /*HAVE_LIBGVC*/
+
 	{ "EditToolkits", 
 		NULL, N_( "_Edit" ), NULL,
 		N_( "Edit toolkits" ), 
-		G_CALLBACK( mainw_program_new_action_cb ) },
-
-	{ "About", 
-		NULL, N_( "_About" ), NULL,
-		N_( "About this program" ), 
-		G_CALLBACK( mainw_about_action_cb ) },
-
-	{ "Guide", 
-		GTK_STOCK_HELP, N_( "_Contents" ), "F1",
-		N_( "Open the users guide" ), 
-		G_CALLBACK( mainw_guide_action_cb ) },
-
-	{ "Homepage", 
-		NULL, N_( "_Website" ), NULL,
-		N_( "Open the VIPS Homepage" ), 
-		G_CALLBACK( mainw_homepage_action_cb ) },
+		G_CALLBACK( mainw_program_new_action_cb ) }
 };
 
 static GtkToggleActionEntry mainw_toggle_actions[] = {
@@ -1801,6 +1837,7 @@ static const char *mainw_menubar_ui_description =
 "      <menuitem action='Recover'/>"
 "      <separator/>"
 "      <menuitem action='Close'/>"
+"      <menuitem action='Quit'/>"
 "    </menu>"
 "    <menu action='EditMenu'>"
 "      <menuitem action='Delete'/>"
@@ -1828,6 +1865,10 @@ static const char *mainw_menubar_ui_description =
 "      <menuitem action='ToolkitBrowser'/>"
 "      <menuitem action='WorkspaceDefs'/>"
 "      <separator/>"
+#ifdef HAVE_LIBGVC
+"      <menuitem action='Graph'/>"
+"      <separator/>"
+#endif /*HAVE_LIBGVC*/
 "      <menuitem action='Normal'/>"
 "      <menuitem action='ShowFormula'/>"
 "      <menuitem action='NoEdit'/>"
@@ -1862,23 +1903,25 @@ static const char *mainw_toolbar_ui_description =
 static void
 mainw_lpane_changed_cb( Pane *pane, Mainw *mainw )
 {
-	mainw->ws->lpane_open = pane->open;
-	mainw->ws->lpane_position = pane->position;
+	if( mainw->ws->lpane_open != pane->open ||
+		mainw->ws->lpane_position != pane->user_position ) {
+		mainw->ws->lpane_open = pane->open;
+		mainw->ws->lpane_position = pane->user_position;
 
-	/* Give menus an update.
-	 */
-	mainw_refresh( mainw );
+		mainw_refresh( mainw );
+	}
 }
 
 static void
 mainw_rpane_changed_cb( Pane *pane, Mainw *mainw )
 {
-	mainw->ws->rpane_open = pane->open;
-	mainw->ws->rpane_position = pane->position;
+	if( mainw->ws->rpane_open != pane->open ||
+		mainw->ws->rpane_position != pane->user_position ) {
+		mainw->ws->rpane_open = pane->open;
+		mainw->ws->rpane_position = pane->user_position;
 
-	/* Give menus an update.
-	 */
-	mainw_refresh( mainw );
+		mainw_refresh( mainw );
+	}
 }
 
 static void
@@ -1900,7 +1943,6 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
         GtkWidget *mbar;
 	GtkWidget *frame;
 	GtkWidget *ebox;
-	GtkAccelGroup *accel_group;
 	GError *error;
 	GtkWidget *cancel;
 	GtkWidget *item;
@@ -1912,46 +1954,36 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 
         /* Make main menu bar
          */
-	mainw->action_group = gtk_action_group_new( "MainwActions" );
-	gtk_action_group_set_translation_domain( mainw->action_group, 
-		GETTEXT_PACKAGE );
-	gtk_action_group_add_actions( mainw->action_group, 
+	gtk_action_group_add_actions( iwnd->action_group, 
 		mainw_actions, G_N_ELEMENTS( mainw_actions ), 
 		GTK_WINDOW( mainw ) );
-	gtk_action_group_add_toggle_actions( mainw->action_group, 
+	gtk_action_group_add_toggle_actions( iwnd->action_group, 
 		mainw_toggle_actions, G_N_ELEMENTS( mainw_toggle_actions ), 
 		GTK_WINDOW( mainw ) );
-	gtk_action_group_add_radio_actions( mainw->action_group,
+	gtk_action_group_add_radio_actions( iwnd->action_group,
 		mainw_radio_actions, G_N_ELEMENTS( mainw_radio_actions ), 
 		WORKSPACE_MODE_REGULAR,
 		G_CALLBACK( mainw_mode_action_cb ),
 		GTK_WINDOW( mainw ) );
 
-	mainw->ui_manager = gtk_ui_manager_new();
-	gtk_ui_manager_insert_action_group( mainw->ui_manager, 
-		mainw->action_group, 0 );
-
-	accel_group = gtk_ui_manager_get_accel_group( mainw->ui_manager );
-	gtk_window_add_accel_group( GTK_WINDOW( mainw ), accel_group );
-
 	error = NULL;
-	if( !gtk_ui_manager_add_ui_from_string( mainw->ui_manager,
+	if( !gtk_ui_manager_add_ui_from_string( iwnd->ui_manager,
 			mainw_menubar_ui_description, -1, &error ) ||
-		!gtk_ui_manager_add_ui_from_string( mainw->ui_manager,
+		!gtk_ui_manager_add_ui_from_string( iwnd->ui_manager,
 			mainw_toolbar_ui_description, -1, &error ) ) {
 		g_message( "building menus failed: %s", error->message );
 		g_error_free( error );
 		exit( EXIT_FAILURE );
 	}
 
-	mbar = gtk_ui_manager_get_widget( mainw->ui_manager, "/MainwMenubar" );
+	mbar = gtk_ui_manager_get_widget( iwnd->ui_manager, "/MainwMenubar" );
 	gtk_box_pack_start( GTK_BOX( vbox ), mbar, FALSE, FALSE, 0 );
         gtk_widget_show( mbar );
 
 	/* Get the dummy item on the recent menu, then get the enclosing menu
 	 * for that dummy item.
 	 */
-        item = gtk_ui_manager_get_widget( mainw->ui_manager,
+        item = gtk_ui_manager_get_widget( iwnd->ui_manager,
 		"/MainwMenubar/FileMenu/RecentMenu/Stub" );
 	mainw->recent_menu = gtk_widget_get_parent( GTK_WIDGET( item ) );
         gtk_signal_connect( GTK_OBJECT( mainw->recent_menu ), "map",
@@ -1959,7 +1991,7 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 
 	/* Same for the column jump menu.
 	 */
-        item = gtk_ui_manager_get_widget( mainw->ui_manager,
+        item = gtk_ui_manager_get_widget( iwnd->ui_manager,
 		"/MainwMenubar/EditMenu/JumpToColumnMenu/Stub" );
 	mainw->jump_to_column_menu = 
 		gtk_widget_get_parent( GTK_WIDGET( item ) );
@@ -1967,7 +1999,7 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	/* Attach toolbar.
   	 */
 	mainw->toolbar = gtk_ui_manager_get_widget( 
-		mainw->ui_manager, "/MainwToolbar" );
+		iwnd->ui_manager, "/MainwToolbar" );
 	gtk_box_pack_start( GTK_BOX( vbox ), mainw->toolbar, FALSE, FALSE, 0 );
         widget_visible( mainw->toolbar, MAINW_TOOLBAR );
 
@@ -2095,9 +2127,6 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	/* Toolkit Browser pane.
 	 */
 	panechild = panechild_new( mainw->rpane, _( "Toolkit Browser" ) );
-	gtk_paned_pack2( GTK_PANED( mainw->rpane ), 
-		GTK_WIDGET( panechild ), TRUE, TRUE );
-	gtk_widget_show( GTK_WIDGET( panechild ) );
 
 	/* Have to put toolkitbrowser in an ebox so the search entry gets
 	 * clipped to the pane size.
@@ -2117,9 +2146,6 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	/* Workspace-local defs pane.
 	 */
 	panechild = panechild_new( mainw->lpane, _( "Workspace Definitions" ) );
-	gtk_paned_pack1( GTK_PANED( mainw->lpane ), 
-		GTK_WIDGET( panechild ), TRUE, TRUE );
-	gtk_widget_show( GTK_WIDGET( panechild ) );
 
 	mainw->workspacedefs = workspacedefs_new( mainw );
 	vobject_link( VOBJECT( mainw->workspacedefs ), IOBJECT( mainw->ws ) );
@@ -2143,10 +2169,22 @@ mainw_popdown( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 {
 	Mainw *mainw = MAINW( iwnd );
 
-	IM_FREEF( g_source_remove, mainw->compat_timeout );
+	/* We can be destroyed in two ways: either our iwnd tells us to go, or
+	 * our model is destroyed under us. If the model has gone, we just go.
+	 * If the model is still there, we need to ask about saving and
+	 * quitting.
+	 */
+	if( mainw->ws ) {
+		/* Argh, cancel this immediately, in case we close before
+		 * we're up properly.
+		 */
+		IM_FREEF( g_source_remove, mainw->compat_timeout );
 
-	filemodel_inter_savenclose_cb( IWINDOW( mainw ), 
-		FILEMODEL( mainw->ws ), nfn, sys );
+		filemodel_inter_savenclose_cb( IWINDOW( mainw ), 
+			FILEMODEL( mainw->ws ), nfn, sys );
+	}
+	else
+		nfn( sys, IWINDOW_YES );
 }
 
 static void
@@ -2163,8 +2201,9 @@ mainw_link( Mainw *mainw, Workspace *ws )
 	iwindow_set_build( IWINDOW( mainw ), 
 		(iWindowBuildFn) mainw_build, ws, NULL, NULL );
 	iwindow_set_popdown( IWINDOW( mainw ), mainw_popdown, NULL );
+	filemodel_set_window_hint( FILEMODEL( ws ), IWINDOW( mainw ) );
 
-	/* If we have a saved size for this workspoace, set that. Otherwise,
+	/* If we have a saved size for this workspace, set that. Otherwise,
 	 * default to the default.
 	 */
 	if( !ws->window_width ) {
@@ -2185,10 +2224,8 @@ mainw_link( Mainw *mainw, Workspace *ws )
 				gdk_screen_get_height( screen ) ) );
 	}
 
-	pane_set_open_position( mainw->lpane, 
-		ws->lpane_open, ws->lpane_position );
-	pane_set_open_position( mainw->rpane, 
-		ws->rpane_open, ws->rpane_position );
+	pane_set_state( mainw->lpane, ws->lpane_open, ws->lpane_position );
+	pane_set_state( mainw->rpane, ws->rpane_open, ws->rpane_position );
 }
 
 Mainw *
