@@ -33,6 +33,10 @@
 #define DEBUG_REGEN
  */
 
+/* trace each reduction
+#define DEBUG_TRACE
+ */
+
 /* trace copies of code from compile heap to main heap.
 #define DEBUG_COPY
  */
@@ -82,8 +86,22 @@ reduce_throw( Reduce *rc )
 		longjmp( rc->error[--rc->running], -1 );
 }
 
+
+static gboolean
+reduce_safe_pointer_wrap( Reduce *rc, 
+	PElement *out, reduce_safe_pointer_fn fn, 
+	void *a, void *b, void *c, void *d,
+	void **result )
+{
+	REDUCE_CATCH_START( FALSE );
+	*result = fn( rc, out, a, b, c, d );
+	REDUCE_CATCH_STOP; 
+
+	return( TRUE );
+}
+
 /* Call a function, passing in a "safe" PElement ... ie. the PElement points
- * at a fresh element which will be safe from the GC.
+ * at a fresh element which will be safe from the GC. 
  */
 void *
 reduce_safe_pointer( Reduce *rc, reduce_safe_pointer_fn fn, 
@@ -98,8 +116,10 @@ reduce_safe_pointer( Reduce *rc, reduce_safe_pointer_fn fn,
 	PEPOINTE( &pe, &e );
 	heap_register_element( rc->heap, &e );
 
-	result = fn( rc, &pe, a, b, c, d );
-
+	if( !reduce_safe_pointer_wrap( rc, &pe, fn, a, b, c, d, &result ) ) {
+		heap_unregister_element( rc->heap, &e );
+		reduce_throw( rc );
+	}
 	heap_unregister_element( rc->heap, &e );
 
 	return( result );
@@ -533,6 +553,12 @@ reduce_get_matrix_size( Reduce *rc,
 	(void) reduce_map_list( rc, base, 
 		(reduce_map_list_fn) reduce_get_line_size, &w, &h );
 
+	if( w == 0 || h == 0 ) {
+		error_top( _( "Zero dimension." ) );
+		error_sub( _( "Matrix has width %d, height %d." ), w, h ); 
+		reduce_throw( rc );
+	}
+
 	*xsize = w;
 	*ysize = h;
 }
@@ -1050,6 +1076,16 @@ reduce_start:
 		}
 	}
 
+#ifdef DEBUG_TRACE
+{
+	char txt[1024];
+	VipsBuf buf = VIPS_BUF_STATIC( txt );
+
+	graph_pelement( rc->heap, &buf, out, TRUE );
+	printf( "reduce_spine: %s\n", vips_buf_all( &buf ) );
+}
+#endif /*DEBUG_TRACE*/
+
 	switch( PEGETTYPE( &np ) ) {
 	case ELEMENT_CHAR:
 	case ELEMENT_BOOL:
@@ -1174,6 +1210,9 @@ reduce_start:
 		case SYM_PARAM:
 			/* All params should be taken out by var abstract.
 			 */
+			printf( "sym-param found, argh: " ); 
+			symbol_name_print( sym );
+			printf( "\n" );
 			g_assert( FALSE );
 			break;
 
