@@ -1023,7 +1023,7 @@ workspace_save( Model *model, xmlNode *xnode )
 	return( xthis );
 }
 
-void
+static void
 workspace_empty( Model *model )
 {
 	Workspace *ws = WORKSPACE( model );
@@ -1280,8 +1280,8 @@ workspace_save_all( Filemodel *filemodel, const char *filename )
 		NN( IOBJECT( filemodel )->name ), filename );
 #endif /*DEBUG*/
 
-	if( (result = FILEMODEL_CLASS( parent_class )->save_all( filemodel,
-		filename )) )
+	if( (result = FILEMODEL_CLASS( parent_class )->
+		save_all( filemodel, filename )) )
 		/* This will add save-as files to recent too. Don't note
 		 * auto_load on recent, since it won't have been loaded by the
 		 * user.
@@ -1944,22 +1944,22 @@ workspace_local_set_from_file( Workspace *ws, const char *fname )
 	iOpenFile *of;
 	char *txt;
 
-	if( !(of = file_open_read( "%s", fname )) ) 
+	if( !(of = ifile_open_read( "%s", fname )) ) 
 		return( FALSE );
-	if( !(txt = file_read( of )) ) {
-		file_close( of );
+	if( !(txt = ifile_read( of )) ) {
+		ifile_close( of );
 		return( FALSE );
 	}
 	if( !workspace_local_set( ws, txt ) ) {
 		g_free( txt );
-		file_close( of );
+		ifile_close( of );
 		return( FALSE );
 	}
 
 	filemodel_set_filename( FILEMODEL( ws->local_kit ), fname );
 
 	g_free( txt );
-	file_close( of );
+	ifile_close( of );
 
 	return( TRUE );
 }
